from api.models import SQLResponse, QueryRequest, TableRequest
from nlu.context_manager import context_manager
from sql_generator.parser import IntentParser
import os
from fastapi import APIRouter, HTTPException
from typing import List, Dict, Optional
import dmPython

router = APIRouter()

# 初始化解析器（单例模式）
_config_path = os.path.join(
    os.path.dirname(os.path.dirname(__file__)),
    "config/intent_config.json"
)

_parser = None


def get_column_comments(cursor, schema: str, table: str) -> Dict[str, str]:
    """
    获取指定表中所有字段的备注信息。
    """
    query = f"""
    SELECT aa.COLUMN_NAME, ss.comments AS 描述
    FROM user_tab_columns aa
    INNER JOIN user_col_comments ss 
        ON ss.COLUMN_NAME = aa.COLUMN_NAME 
        AND ss.Table_Name = aa.Table_Name
    WHERE aa.Table_Name = '{table}'
    ORDER BY aa.column_id
    """
    cursor.execute(query)
    return {row[0]: row[1] if row[1] else row[0] for row in cursor.fetchall()}


def get_parser():
    global _parser
    if not _parser:
        try:
            _parser = IntentParser(_config_path)
        except Exception as e:
            raise RuntimeError(f"配置加载失败: {str(e)}")
    return _parser


@router.post("/generate-sql", response_model=SQLResponse)
async def generate_sql(query: QueryRequest):
    result_list = []
    try:
        # 获取或创建会话
        session_id = query.session_id or context_manager.create_context()
        ctx = context_manager.get_context(session_id) or {}

        # 解析查询
        parser = get_parser()
        sql = parser.parse_with_context(query.question, ctx)

        from_pos = sql.lower().find('from') + len('from')  # 'from' 后的第一个字符的位置

        # 查找 'where' 的位置，如果没有找到则返回 -1
        where_pos = sql.lower().find('where')

        if where_pos != -1:
            # 如果存在 'where' 子句，则提取 'from' 和 'where' 中间的内容，并去除空格
            table_name = sql[from_pos:where_pos].strip()
        else:
            # 如果没有 'where' 子句，则提取 'from' 后面的所有内容，并去除空格
            table_name = sql[from_pos:].strip()

        try:
            conn = dmPython.connect(user="SYSDBA", password="SYSDBA123", server="127.0.0.1", port=5236,
                                    schema='lqbz')
            cursor = conn.cursor()
            try:
                # 获取字段备注信息
                column_comments = get_column_comments(cursor, schema='lqbz', table=table_name)

                # 执行查询
                cursor.execute(sql)

                # 获取列名
                column_names = [description[0] for description in cursor.description]

                # 获取所有数据
                rows = cursor.fetchall()

                # 将每一行数据与列名组合成字典，并使用备注信息作为键
                result_dicts = [
                    {column_comments.get(column_name, column_name): value for column_name, value in zip(column_names, row)}
                    for row in rows]

                # 收集结果到列表中
                for record_dict in result_dicts:
                    result_list.append(record_dict)

            except (dmPython.Error, Exception) as err:
                print(f"发生错误: {err}")
            finally:
                cursor.close()  # 确保在结束时关闭游标
        except (dmPython.Error, Exception) as err:
            print(f"发生错误: {err}")
        finally:
            if 'conn' in locals():
                conn.close()  # 确保在结束时关闭连接

        # 更新上下文
        # if new_ctx:
        #     context_manager.update_context(session_id, **new_ctx)

        return SQLResponse(
            success=bool(sql),
            sql=sql,
            session_id=session_id,
            error="无法解析查询" if not sql else None,
            result_list=result_list  # 返回收集的结果列表
        )

    except Exception as e:
        return SQLResponse(
            success=False,
            error=str(e),
            session_id=query.session_id,
            result_list=result_list  # 即使有错误也返回空的结果列表
        )