from fastapi import FastAPI, HTTPException, Request
from pydantic import BaseModel
import aiohttp
import logging
from fastapi.responses import JSONResponse
import json
from typing import Optional, List, Dict

app = FastAPI()

# 配置日志
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


class Query(BaseModel):
    question: str
    userId: str


async def call_generate_sql(question: str, userId: str) -> Optional[List]:
    url = "http://127.0.0.1:8000/api/v1/generate-sql"
    payload = {
        "userId": userId,
        "session_id": None,
        "question": question
    }
    try:
        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=payload) as response:
                if response.status == 200:
                    data = await response.json()
                    # 返回实际的数据
                    return data
                else:
                    logger.error(f"Error calling generate-sql service: {response.status} - {await response.text()}")
                    return None
    except Exception as e:
        logger.error(f"Unexpected error while calling generate-sql service: {e}")
        return None


async def call_rasa(question: str, userId: str) -> dict:
    url = "http://localhost:5005/webhooks/rest/webhook"  # Rasa 默认的 webhook 端点
    payload = {
        "userId": userId,
        "message": question
    }
    try:
        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=payload) as response:
                if response.status == 200:
                    content_type = response.headers.get("Content-Type", "")
                    if "application/json" in content_type:
                        data = await response.json()
                        logger.info(f"Response from Rasa: {data}")

                        # 判断是否是简单的文本回复
                        if (isinstance(data, list) and len(data) == 1 and
                                isinstance(data[0], dict) and
                                'recipient_id' in data[0] and
                                'text' in data[0]):
                            # 这里只返回 text 的内容
                            return {
                                "code": 200,
                                "msg": "操作成功",
                                "data": {
                                    "userId": userId,
                                    "answer": data[0].get("text"),
                                    "ossId": None,
                                    "fileName": None,
                                    "filePage": None,
                                    "graph": None
                                }
                            }
                        answer = data[0].get("text", "No answer found")
                        entity_lists = data[1].get("text", "No graph found")[1:-1] if len(data) > 1 else "None"
                        entity_str = entity_lists.replace("'", '"')
                        try:
                            entity_data = json.loads(entity_str)
                            doc = entity_data['docs'][0]
                            ossid = doc['ossId']
                            file_name = doc['file_name']
                            filePage = doc["filePage"]
                            graph = {
                                "data": entity_data['data'],
                                "links": entity_data['links']
                            }
                            return {
                                "code": 200,
                                "msg": "操作成功",
                                "data": {
                                    "userId": userId,
                                    "answer": answer,
                                    "ossId": str(ossid),
                                    "fileName": file_name,
                                    "filePage": filePage,
                                    "graph": str(graph)
                                }
                            }
                        except (json.JSONDecodeError, KeyError, IndexError):
                            logger.error("Failed to parse additional data from Rasa response.")

                        # 如果没有额外的数据或解析失败，则只返回基本的回答
                        return {
                            "code": 200,
                            "msg": "操作成功",
                            "data": {
                                "userId": userId,
                                "answer": answer,
                                "ossId": None,
                                "fileName": None,
                                "filePage": None,
                                "graph": None
                            }
                        }

                    else:
                        logger.error(f"Invalid content type: {content_type}")
                        raise HTTPException(status_code=500, detail="Invalid response content type")
                else:
                    logger.error(f"Error calling Rasa server: {response.status} - {await response.text()}")
                    raise HTTPException(status_code=response.status, detail="Error while calling Rasa Server")
    except Exception as e:
        logger.error(f"Unexpected error while calling Rasa server: {e}")
        raise HTTPException(status_code=500, detail="Internal Server Error")


@app.post("/kgqa/ask")
async def ask_rasa(query: Query):
    response_from_generate_sql = await call_generate_sql(query.question, query.userId)

    if response_from_generate_sql == {
        "result_list": []
    }:
        # 如果 generate-sql 返回的是空列表，则继续调用 Rasa
        return await call_rasa(query.question, query.userId)
    elif response_from_generate_sql is not None:
        # 如果 generate-sql 返回了非空列表，则直接返回这些数据
        return {"code": 200, "msg": "Data from generate-sql service", "data": response_from_generate_sql}
    else:
        # 如果 generate-sql 调用失败，则调用 Rasa
        return await call_rasa(query.question, query.userId)


@app.exception_handler(HTTPException)
async def http_exception_handler(request: Request, exc: HTTPException):
    return JSONResponse(
        status_code=exc.status_code,
        content={"detail": exc.detail},
    )


if __name__ == "__main__":
    import uvicorn

    uvicorn.run(app, host="0.0.0.0", port=7074)