import json
import re
from typing import Optional


def _load_config(config_path):
    try:
        with open(config_path, 'r', encoding='utf-8') as f:
            config_data = json.load(f)
        return config_data['intent_mapping']
    except (FileNotFoundError, json.JSONDecodeError) as e:
        raise ValueError(f"配置文件错误: {str(e)}")


def _extract_conditions(question, intent):
    conditions = []
    # 优先使用正则表达式匹配
    for col_name, regx_info in intent['regx_map'].items():
        # 双重匹配：既有注释关键词又匹配正则
        if any(comment in question for comment in regx_info['comments']):
            match = regx_info['pattern'].search(question)
            if match:
                value = match.group(1)
                conditions.append(f"{col_name} = '{value}'")
    # 补充匹配未配置正则的机号
    if not any(c.startswith('aircraft_no') or c.startswith('air_no') for c in conditions):
        aircraft_match = re.search(r'(\d+)号', question)
        aircraft_col = next((v for k, v in intent['comment_map'].items()
                             if k in ['机号', '编号', '飞机编号']), None)
        if aircraft_col and aircraft_match:
            conditions.append(f"{aircraft_col} = {aircraft_match.group(1)}")

    return conditions


def parse(question: str, intent) -> Optional[str]:
    try:
        if not intent:
            return None

        # 处理聚合查询
        count_needed = any(kw in question for kw in ['数量', '多少'])

        # 构建SELECT子句
        if count_needed:
            select_clause = "COUNT(1)"
        else:
            selected_columns = []
            for col in intent['columns']:
                if any(comment in question for comment in col['comment']):
                    selected_columns.append(col['column'])
            select_clause = "*" if not selected_columns else ", ".join(selected_columns)

        # 构建WHERE条件
        # conditions = _extract_conditions(question, intent)
        # 组合SQL
        # where_clause = f" WHERE {' AND '.join(conditions)}" if conditions else ""

        return select_clause
        # f"SELECT {select_clause} FROM {intent['table']}{where_clause}"
    except Exception as e:
        raise RuntimeError(f"解析失败: {str(e)}")


class IntentParser:
    def __init__(self, config_path):
        self.intents = _load_config(config_path)
        self._preprocess_intents()

    def _preprocess_intents(self):
        for intent in self.intents:
            intent['comment_map'] = {}
            intent['regx_map'] = {}
            intent['all_comments'] = []
            for col in intent['columns']:
                # 构建注释映射
                for comment in col.get('comment', []):
                    intent['comment_map'][comment] = col['column']
                    intent['all_comments'].append(comment)
                # 构建正则表达式映射
                if 'regx' in col:
                    try:
                        compiled_regx = re.compile(col['regx'])
                    except re.error as e:
                        raise ValueError(f"无效正则表达式: {col['regx']} 错误: {str(e)}")
                    intent['regx_map'][col['column']] = {
                        'pattern': compiled_regx,
                        'comments': col.get('comment', [])
                    }
            intent['description_keywords'] = intent['description']

    def _find_intent(self, question):
        # 先通过description匹配
        for intent in self.intents:
            for keyword in intent['description_keywords']:
                if keyword in question:
                    return intent
        # 次通过列注释匹配
        max_score = 0
        best_intent = None
        for intent in self.intents:
            score = sum(1 for comment in intent['all_comments'] if comment in question)
            if score > max_score:
                max_score = score
                best_intent = intent
        return best_intent

    def parse_with_context(self, question: str, context: dict) -> tuple:
        """返回（SQL语句，新上下文）"""
        # 尝试正常解析
        intent = self._find_intent(question)

        # 上下文增强解析
        if not intent and context["last_table"]:
            intent = self._get_intent_by_table(context["last_table"])

        if not intent:
            return None, {}

        # 提取机号
        aircraft_col = next((col["column"] for col in intent["columns"]
                             if any(c in ["机号", "飞机编号"] for c in col["comment"])), None)

        # 解析条件
        conditions = _extract_conditions(question, intent)

        # 上下文条件合并
        if context["aircraft_column"] == aircraft_col:
            # 保留原有非机号条件
            keep_conds = [c for c in context["base_conditions"].values()
                          if not c.startswith(aircraft_col)]
            conditions += keep_conds

        # 构建SQL
        select_clause = parse(question, intent)
        where_clause = f" WHERE {' AND '.join(conditions)}" if conditions else ""

        # 更新上下文
        # new_context = {
        #     "last_table": intent["table"],
        #     "aircraft_column": aircraft_col,
        #     "base_conditions": {cond.split()[0]: cond for cond in conditions
        #                         if not cond.startswith(aircraft_col)}
        # }

        return f"SELECT {select_clause} FROM {intent['table']}{where_clause}"

    def _get_intent_by_table(self, table_name):
        return next((i for i in self.intents if i["table"] == table_name), None)